/* BDF - Copyright (c) 2015-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi.commands;

import fr.exemole.bdfserver.multi.Multi;
import fr.exemole.bdfserver.multi.MultiConf;
import fr.exemole.bdfserver.multi.MultiUtils;
import fr.exemole.bdfserver.storage.directory.bdfdata.StartValues;
import net.fichotheque.SubsetKey;
import net.fichotheque.utils.SphereUtils;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class FichothequeCreationCommand {

    public final static String COMMAND_NAME = "FichothequeCreation";
    public final static String NEWFICHOTHEQUE_PARAMNAME = "newfichotheque";
    public final static String FIRSTSPHERE_PARAMNAME = "firstsphere";
    public final static String FIRSTUSER_PARAMNAME = "firstuser";
    public final static String FIRSTPASSWORD1_PARAMNAME = "firstpassword_1";
    public final static String FIRSTPASSWORD2_PARAMNAME = "firstpassword_2";

    private final Multi multi;
    private final RequestMap requestMap;

    public FichothequeCreationCommand(Multi multi, RequestMap requestMap) {
        this.multi = multi;
        this.requestMap = requestMap;
    }

    public CommandMessage doCommand() {
        try {
            String name = run();
            return LogUtils.done("_ done.multi.fichothequecreation", name);
        } catch (ErrorMessageException eme) {
            return eme.getErrorMessage();
        }
    }

    private String run() throws ErrorMessageException {
        MultiConf multiConf = multi.getMultiConf();
        String name = MultiUtils.getFichothequeName(requestMap, NEWFICHOTHEQUE_PARAMNAME);
        if (multi.getExistingNameSet().contains(name)) {
            throw new ErrorMessageException("_ error.existing.fichotheque", name);
        }
        StartValues startValues = getStartValues(requestMap)
                .authority(multiConf.getAuthority())
                .basename(name)
                .lang(multiConf.getLang().toString());
        multi.createFichotheque(CreationParameters.initNew(name, startValues));
        return name;
    }


    public static StartValues getStartValues(RequestMap requestMap) throws ErrorMessageException {
        String sphereName = MultiUtils.getMandatory(requestMap, FIRSTSPHERE_PARAMNAME);
        String firstUser = MultiUtils.getMandatory(requestMap, FIRSTUSER_PARAMNAME);
        String password1 = MultiUtils.getMandatory(requestMap, FIRSTPASSWORD1_PARAMNAME);
        String password2 = MultiUtils.getMandatory(requestMap, FIRSTPASSWORD2_PARAMNAME);
        SubsetKey sphereKey;
        try {
            sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereName);
        } catch (java.text.ParseException pe) {
            throw new ErrorMessageException("_ error.wrong.spherename", sphereName);
        }
        if (!SphereUtils.testLogin(firstUser)) {
            throw new ErrorMessageException("_ error.wrong.login", firstUser);
        }
        if (!password1.equals(password2)) {
            throw new ErrorMessageException("_ error.wrong.password_different");
        }
        return StartValues.init()
                .spherename(sphereKey.getSubsetName())
                .firstuser(firstUser)
                .firstpassword(password1);
    }

}
