/* BdfServer - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools;

import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public class BdfHref implements CharSequence {

    private final StringBuilder buf = new StringBuilder();
    private boolean first = true;

    public BdfHref(String href, boolean first) {
        buf.append(href);
        this.first = first;
    }

    public BdfHref page(String pageName) {
        testFirst();
        buf.append("page=");
        buf.append(pageName);
        return this;
    }

    public BdfHref pageErr(String pageName) {
        testFirst();
        buf.append("page-err=");
        buf.append(pageName);
        return this;
    }

    public BdfHref stream(String streamName) {
        testFirst();
        buf.append("stream=");
        buf.append(streamName);
        return this;
    }

    public BdfHref command(String commandName) {
        testFirst();
        buf.append("cmd=");
        buf.append(commandName);
        return this;
    }

    public BdfHref param(String paramName, String paramValue) {
        testFirst();
        buf.append(paramName);
        buf.append("=");
        buf.append(paramValue);
        return this;
    }

    public BdfHref sqlExport(String sqlExportName) {
        testFirst();
        buf.append(InteractionConstants.SQLEXPORT_PARAMNAME);
        buf.append("=");
        buf.append(sqlExportName);
        return this;
    }

    public BdfHref scrutariExport(String scrutariExportName) {
        testFirst();
        buf.append(InteractionConstants.SCRUTARIEXPORT_PARAMNAME);
        buf.append("=");
        buf.append(scrutariExportName);
        return this;
    }

    public BdfHref tableExport(String tableExportName) {
        testFirst();
        buf.append(InteractionConstants.TABLEEXPORT_PARAMNAME);
        buf.append("=");
        buf.append(tableExportName);
        return this;
    }

    public BdfHref balayage(String balayageName) {
        testFirst();
        buf.append(InteractionConstants.BALAYAGE_PARAMNAME);
        buf.append("=");
        buf.append(balayageName);
        return this;
    }

    public BdfHref refresh() {
        testFirst();
        buf.append(InteractionConstants.REFRESH_PARAMNAME);
        buf.append("=");
        buf.append(System.currentTimeMillis());
        return this;
    }

    public BdfHref subset(Subset subset) {
        return subset(subset.getSubsetKey());
    }

    public BdfHref subset(SubsetKey subsetKey) {
        testFirst();
        buf.append(subsetKey.getCategoryString());
        buf.append("=");
        buf.append(subsetKey.getSubsetName());
        return this;
    }

    public BdfHref subsetItem(SubsetItem subsetItem) {
        SubsetKey subsetKey = subsetItem.getSubset().getSubsetKey();
        subset(subsetKey);
        buf.append('&');
        buf.append(InteractionConstants.ID_PARAMNAME);
        buf.append("=");
        buf.append(subsetItem.getId());
        return this;
    }

    public BdfHref derive(String paramName, String paramValue) {
        BdfHref newInstance = new BdfHref(buf.toString(), first);
        return newInstance.param(paramName, paramValue);
    }

    @Override
    public String toString() {
        return buf.toString();
    }

    @Override
    public int length() {
        return buf.length();
    }

    @Override
    public char charAt(int index) {
        return buf.charAt(index);
    }

    @Override
    public CharSequence subSequence(int start, int end) {
        return buf.subSequence(start, end);
    }

    private void testFirst() {
        if (first) {
            buf.append('?');
            first = false;
        } else {
            buf.append('&');
        }
    }

}
