/* BdfServer_Commands - Copyright (c) 2015-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.addenda;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import java.text.ParseException;
import net.fichotheque.ExistingSubsetException;
import net.fichotheque.SubsetKey;
import net.fichotheque.addenda.AddendaEditor;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class AddendaCreationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "AddendaCreation";
    public final static String COMMANDKEY = "_ ADD-01";
    public final static String NEWADDENDA_PARAMNAME = "newaddenda";
    private SubsetKey newSubsetKey;

    public AddendaCreationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.ADDENDA + "/" + COMMANDNAME);
        try {
            AddendaEditor addendaEditor = editSession.getFichothequeEditor().createAddenda(newSubsetKey);
            setDone("_ done.addenda.addendacreation", newSubsetKey.toString());
            putResultObject(ADDENDA_OBJ, addendaEditor.getAddenda());
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test done before", ese);
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdminPermission();
        String newAddendaName = getMandatory(NEWADDENDA_PARAMNAME);
        newAddendaName = newAddendaName.trim();
        if (newAddendaName.length() == 0) {
            setError("_ error.empty.addendaname");
            return false;
        }
        try {
            newSubsetKey = SubsetKey.parse(SubsetKey.CATEGORY_ADDENDA, newAddendaName);
        } catch (ParseException pe) {
            setError("_ error.wrong.addendaname", newAddendaName);
            return false;
        }
        if (fichotheque.getSubset(newSubsetKey) != null) {
            setError("_ error.existing.addenda", newAddendaName);
            return false;
        }
        return true;
    }

}
