/* BdfServer_Commands - Copyright (c) 2015-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.addenda;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import net.fichotheque.FichothequeEditor;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.AddendaEditor;
import net.fichotheque.addenda.Document;
import net.fichotheque.tools.FichothequeTools;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class DocumentRemoveCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "DocumentRemove";
    public final static String COMMANDKEY = "_ ADD-06";
    private Document document;

    public DocumentRemoveCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.ADDENDA + "/" + COMMANDNAME);
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        FichothequeTools.prepareToRemove(fichothequeEditor, document);
        Addenda addenda = document.getAddenda();
        AddendaEditor addendaEditor = fichothequeEditor.getAddendaEditor(addenda.getSubsetKey());
        addendaEditor.removeDocument(document);
        setDone("_ done.addenda.documentremove");
        putResultObject(ADDENDA_OBJ, addenda);
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        document = getMandatoryDocument();
        PermissionChecker.init(bdfServer, bdfUser)
                .checkWrite(document);
        if (!checkConfirmation()) {
            setError("_ error.empty.confirmationcheck_remove");
            return false;
        }
        return true;
    }

}
