/* BdfServer_Commands - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.album;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.commands.AbstractPhrasesCommand;
import net.fichotheque.album.Album;
import net.fichotheque.album.metadata.AlbumMetadataEditor;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class AlbumPhrasesCommand extends AbstractPhrasesCommand {

    public final static String COMMANDNAME = "AlbumIntitule";
    public final static String COMMANDKEY = "_ ALB-02";
    private Album album;

    public AlbumPhrasesCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.ALBUM + "/" + COMMANDNAME);
        AlbumMetadataEditor albumMetadataEditor = editSession.getFichothequeEditor().getAlbumEditor(album.getSubsetKey()).getAlbumMetadataEditor();
        boolean done = false;
        if (update(albumMetadataEditor)) {
            done = true;
        }
        if (done) {
            setDone("_ done.album.albumphrases");
        }
        putResultObject(ALBUM_OBJ, album);
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        album = getMandatoryAlbum();
        checkSubsetAdminPermission(album);
        boolean done = checkPhrasesParameters();
        return done;
    }

}
