/* BdfServer_Commands - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.album;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.tools.configuration.ConfigurationUtils;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import net.fichotheque.album.Album;
import net.fichotheque.album.AlbumConstants;
import net.fichotheque.album.Illustration;
import net.fichotheque.utils.AlbumUtils;
import net.mapeadores.util.images.ImagesUtils;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class ToolCropCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "ToolCrop";
    public final static String TMPFILE_PARAMNAME = "tmpfile";
    public final static String X_PARAMNAME = "x";
    public final static String Y_PARAMNAME = "y";
    public final static String WIDTH_PARAMNAME = "width";
    public final static String HEIGHT_PARAMNAME = "height";
    private Album album;
    private File tmpFile;
    private Illustration illustration;
    private int x, y, width, height;
    private String extension;

    public ToolCropCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return false;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        File newTmpFile = AlbumCommandUtils.getTmpFile(bdfServer, extension);
        try (InputStream is = getInputStream(); OutputStream os = new FileOutputStream(newTmpFile)) {
            ImagesUtils.cropImage(is, extension, os, x, y, width, height);
            AlbumCommandUtils.minimizeTmpFile(newTmpFile, album);
        } catch (IOException ioe) {
            setError("_ error.exception.io", ioe.getMessage());
            return;
        }
        putResultObject(TMPFILE_OBJ, newTmpFile);
        putResultObject(ALBUM_OBJ, album);
        setDone("_ done.album.toolcrop", newTmpFile.getName());
    }

    private InputStream getInputStream() throws IOException {
        if (tmpFile != null) {
            return new FileInputStream(tmpFile);
        } else {
            return illustration.getInputStream(AlbumConstants.ORIGINAL_SPECIALDIM);
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        album = getMandatoryAlbum();
        x = getMandatoryInteger(X_PARAMNAME);
        y = getMandatoryInteger(Y_PARAMNAME);
        width = getMandatoryInteger(WIDTH_PARAMNAME);
        height = getMandatoryInteger(HEIGHT_PARAMNAME);
        String tmpFileName = requestMap.getParameter(TMPFILE_PARAMNAME);
        if ((tmpFileName != null) && (!tmpFileName.equals(AlbumCommandUtils.ORIGINAL_TMPFILENAME))) {
            File tmpDirectory = ConfigurationUtils.getTmpDirectory(bdfServer);
            tmpFile = new File(tmpDirectory, tmpFileName);
            if (!tmpFile.exists()) {
                this.setError("_ error.unknown.tmpfile", tmpFileName);
                return false;
            }
            extension = AlbumUtils.getExtension(tmpFile);
        } else {
            illustration = BdfInstructionUtils.getMandatoryIllustration(requestMap, album);
            extension = AlbumUtils.formatTypeToString(illustration.getFormatType());
        }
        return true;
    }

}
