/* BdfServer_Commands - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.configuration;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.groups.GroupDef;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class GroupRemoveCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "GroupRemove";
    public final static String COMMANDKEY = "_ CNF-12";
    public final static String NAME_PARAMNAME = "name";
    private GroupDef groupDef;

    public GroupRemoveCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        BdfServerEditor bdfServerEditor = bdfServer.getBdfServerEditor();
        String name = groupDef.getName();
        bdfServerEditor.removeGroupDef(groupDef);
        bdfServerEditor.saveChanges();
        setDone("_ done.configuration.groupremove", name);

    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdminPermission();
        String name = getMandatory(NAME_PARAMNAME);
        groupDef = bdfServer.getGroupManager().getGroupDef(name);
        if (groupDef == null) {
            throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(NAME_PARAMNAME, name));
        }
        if (!checkConfirmation()) {
            setError("_ error.empty.confirmationcheck_remove");
            return false;
        }
        return true;
    }

}
