/* BdfServer_Commands - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import net.fichotheque.FichothequeEditor;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.tools.parsers.AttributeParser;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class CorpusAttributeChangeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "CorpusAttributeChange";
    public final static String COMMANDKEY = "_ CRP-03";
    public final static String ATTRIBUTES_PARAMNAME = "attributes";
    private Corpus corpus;
    private AttributeChange attributeChange;

    public CorpusAttributeChangeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.CORPUS + "/" + COMMANDNAME);
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        CorpusMetadata metadata = corpus.getCorpusMetadata();
        FichothequeUtils.changeAttributes(fichothequeEditor, metadata, attributeChange);
        putResultObject(CORPUS_OBJ, corpus);
        setDone("_ done.global.attributechange");
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        corpus = getMandatoryCorpus();
        checkSubsetAdminPermission(corpus);
        String attributes = getMandatory(ATTRIBUTES_PARAMNAME);
        this.attributeChange = AttributeParser.parse(attributes);
        return true;
    }

}
