/* BdfServer_Commands - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import java.text.ParseException;
import net.fichotheque.ExistingSubsetException;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.CorpusEditor;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class CorpusCreationCommand extends AbstractCorpusCreationCommand {

    public final static String COMMANDNAME = "CorpusCreation";
    public final static String COMMANDKEY = "_ CRP-01";
    public final static String NEWCORPUS_PARAMNAME = "newcorpus";
    private SubsetKey newCorpusKey;

    public CorpusCreationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.CORPUS + "/" + COMMANDNAME);
        try {
            CorpusEditor corpusEditor = editSession.getFichothequeEditor().createCorpus(newCorpusKey, masterSubset);
            putResultObject(CORPUS_OBJ, corpusEditor.getCorpus());
            setDone("_ done.corpus.corpuscreation", newCorpusKey.toString());
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdminPermission();
        String newCorpusName = getMandatory(NEWCORPUS_PARAMNAME);
        initSubsetMaster();
        newCorpusName = newCorpusName.trim();
        if (newCorpusName.length() == 0) {
            setError("_ error.empty.corpusname");
            return false;
        }
        try {
            newCorpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, newCorpusName);
            if (bdfServer.getFichotheque().getSubset(newCorpusKey) != null) {
                setError("_ error.existing.corpus", newCorpusKey.toString());
                return false;
            }
        } catch (ParseException pe) {
            setError("_ error.wrong.corpusname", newCorpusName);
            return false;
        }
        return true;
    }


}
