/* BdfServer_Commands - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadataEditor;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.corpus.metadata.MandatoryFieldException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class FieldRemoveCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "FieldRemove";
    public final static String COMMANDKEY = "_ CRP-08";
    public final static String FIELDKEY_PARAMNAME = "fieldkey";
    private Corpus corpus;
    private CorpusField corpusField;

    public FieldRemoveCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        initEditSession(Domains.CORPUS + "/" + COMMANDNAME);
        putResultObject(CORPUS_OBJ, corpus);
        CorpusMetadataEditor corpusMetadataEditor = editSession.getFichothequeEditor().getCorpusEditor(corpus.getSubsetKey()).getCorpusMetadataEditor();
        try {
            String key = corpusField.getFieldKey().toString();
            corpusMetadataEditor.removeCorpusField(corpusField);
            setDone("_ done.corpus.fieldremove", key);
        } catch (MandatoryFieldException mfe) {
            throw new ShouldNotOccurException("test done before");
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        corpus = getMandatoryCorpus();
        checkSubsetAdminPermission(corpus);
        String fieldKeyValue = getMandatory(FIELDKEY_PARAMNAME);
        fieldKeyValue = fieldKeyValue.trim();
        if (fieldKeyValue.length() == 0) {
            setError("_ error.empty.fieldkey");
            return false;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(fieldKeyValue);
        } catch (ParseException pe) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(FIELDKEY_PARAMNAME, fieldKeyValue));
        }
        corpusField = corpus.getCorpusMetadata().getCorpusField(fieldKey);
        if ((corpusField == null) || (fieldKey.isMandatoryFieldKey())) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(FIELDKEY_PARAMNAME, fieldKeyValue));
        }
        if (!checkConfirmation()) {
            setError("_ error.empty.confirmationcheck_remove");
            return false;
        }
        return true;
    }

}
