/* BdfServer_Commands - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.edition;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.EditionEngine;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import net.fichotheque.ExistingIdException;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.CorpusEditor;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.NoMasterIdException;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class FicheCreationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "FicheCreation";
    private Corpus corpus;

    public FicheCreationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.EDITION + "/" + COMMANDNAME);
        CorpusEditor corpusEditor = editSession.getFichothequeEditor().getCorpusEditor(corpus.getSubsetKey());
        FicheMeta ficheMeta = null;
        try {
            ficheMeta = corpusEditor.createFiche(-1);
        } catch (ExistingIdException eii) {
            throw new ImplementationException(eii);
        } catch (NoMasterIdException nmie) {
            throw new ShouldNotOccurException(nmie);
        }
        corpusEditor.setDate(ficheMeta, FuzzyDate.current(), false);
        EditionEngine.replace(editSession, bdfUser, getPermissionSummary(), requestMap, ficheMeta);
        setDone("_ done.edition.fichecreation", CorpusMetadataUtils.getFicheTitle(ficheMeta, bdfUser.getWorkingLang(), bdfUser.getFormatLocale()));
        putResultObject(CORPUS_OBJ, corpus);
        putResultObject(FICHEMETA_OBJ, ficheMeta);
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        corpus = getMandatoryCorpus();
        if (corpus.getMasterSubset() != null) {
            throw new BdfInstructionException("corpus.getMasterSubset() != null");
        }
        PermissionChecker.init(bdfServer, bdfUser)
                .checkFicheCreate(corpus);
        return true;
    }

}
