/* BdfServer_Commands - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.importation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.tools.importation.ImportationEngine;
import fr.exemole.bdfserver.tools.importation.ImportationFileUtils;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.io.File;
import java.io.IOException;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.tools.importation.CorpusImportBuilder;
import net.fichotheque.tools.importation.ImportationUtils;
import net.fichotheque.tools.importation.ParseResultBuilder;
import net.fichotheque.tools.importation.directory.DirectoryCorpusImport;
import net.fichotheque.tools.importation.parsers.Row;
import net.fichotheque.tools.importation.parsers.RowParser;
import net.fichotheque.tools.parsers.FicheParser;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class CorpusImportParseCommand extends AbstractImportParseCommand {

    public final static String COMMANDNAME = "CorpusImportParse";
    public final static String COMMANDKEY = "_ IMP-01";
    public final static String TYPE_PARAMNAME = "type";
    public final static String CORPUS_PARAMNAME = "corpus";
    private String type;
    private Corpus corpus;


    public CorpusImportParseCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return false;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        FicheParser.Parameters ficheParserParameters = new FicheParser.Parameters(bdfUser, bdfServer.getL10nManager().getCodeCatalog(), BdfUserUtils.getContentChecker(bdfServer, bdfUser), BdfUserUtils.getTypoOptions(bdfUser));
        ficheParserParameters.setUserRedacteur(bdfUser.getRedacteur());
        CorpusImportBuilder corpusImportBuilder = CorpusImportBuilder.newInstance(BdfUserUtils.getContentChecker(bdfServer, bdfUser), corpus, type);
        ParseResultBuilder parseResultBuilder = new ParseResultBuilder(ImportationEngine.CORPUS_IMPORT);
        RowParser rowParser = RowParser.newCorpusRowParser(fieldsArray, corpusImportBuilder, parseResultBuilder, ficheParserParameters, bdfServer.getPolicyManager().getDynamicEditPolicies(), bdfServer.getThesaurusLangChecker());
        if (rowParser != null) {
            for (int i = 0; i < lines.length; i++) {
                String[] line = lines[i];
                if (line.length != 0) {
                    rowParser.parseRow(i + 1, new Row(line));
                }
            }
            try {
                File tmpDir = ImportationFileUtils.createTmpDir(bdfServer, "corpusimport-" + type);
                DirectoryCorpusImport.save(BdfUserUtils.getContentChecker(bdfServer, bdfUser), corpusImportBuilder.toCorpusImport(), tmpDir);
                parseResultBuilder.setTmpDir(tmpDir);
            } catch (IOException ioe) {
                setError("_ error.exception.io", ioe.getMessage());
            }
        }
        putResultObject(PARSERESULT_OBJ, parseResultBuilder.toParseResult());
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdminPermission();
        this.type = getMandatory(TYPE_PARAMNAME);
        if (!ImportationUtils.testCorpusImportType(type)) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(TYPE_PARAMNAME, type));
        }
        this.corpus = (Corpus) BdfInstructionUtils.getMandatorySubset(fichotheque, requestMap, SubsetKey.CATEGORY_CORPUS, CORPUS_PARAMNAME);
        return checkLines();
    }


}
