/* BdfServer_Commands - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.sphere;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.roles.PermissionSummaryBuilder;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.SphereEditor;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurActiveCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "RedacteurActive";
    public final static String COMMANDKEY = "_ SPH-05";
    public static final String ACTIVE_PARAMNAME = "active";
    private Redacteur redacteur;
    private boolean newActive;

    public RedacteurActiveCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.SPHERE + "/" + COMMANDNAME);
        SphereEditor sphereEditor = editSession.getFichothequeEditor().getSphereEditor(redacteur.getSphere().getSubsetKey());
        boolean done = sphereEditor.setActive(redacteur, newActive);
        if (done) {
            setDone("_ done.sphere.redacteuractive");
        }
        putResultObject(SPHERE_OBJ, redacteur.getSphere());
        putResultObject(REDACTEUR_OBJ, redacteur);
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        redacteur = getMandatoryRedacteur();
        checkSubsetAdminPermission(redacteur.getSphere());
        newActive = requestMap.isTrue(ACTIVE_PARAMNAME);
        PermissionSummary redacteurPermissionSummary = PermissionSummaryBuilder.build(bdfServer.getPermissionManager(), redacteur);
        if (redacteurPermissionSummary.isSubsetAdmin(redacteur.getSphere().getSubsetKey())) {
            setError("_ error.unsupported.inactiveadmin");
            return false;
        }
        return true;
    }

}
