/* BdfServer_Commands - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.sphere;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.text.ParseException;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.SphereEditor;
import net.mapeadores.util.models.EmailCore;
import net.mapeadores.util.models.EmailCoreUtils;
import net.mapeadores.util.models.PersonCore;
import net.mapeadores.util.models.PersonCoreUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurChangeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "RedacteurChange";
    public final static String COMMANDKEY = "_ USR-01";
    public final static String SURNAME_PARAMNAME = "surname";
    public final static String NONLATIN_PARAMNAME = "nonlatin";
    public final static String FORENAME_PARAMNAME = "forename";
    public final static String ADDRSPEC_PARAMNAME = "addrspec";
    public final static String SURNAMEFIRST_PARAMNAME = "surnamefirst";
    private PersonCore personCore;
    private EmailCore emailCore;
    private Redacteur redacteur;

    public RedacteurChangeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.SPHERE + "/" + COMMANDNAME);
        SphereEditor sphereEditor = editSession.getFichothequeEditor().getSphereEditor(redacteur.getSphere().getSubsetKey());
        sphereEditor.setPersonCore(redacteur, personCore);
        sphereEditor.setEmailCore(redacteur, emailCore);
        setDone("_ done.sphere.redacteurchange");
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        redacteur = getMandatoryRedacteur();
        boolean isSame = BdfUserUtils.isSame(bdfUser, redacteur);
        if ((!isSame) || (!bdfServer.getPolicyManager().getUserAllow().isDataChangeAllowed())) {
            checkSubsetAdminPermission(redacteur.getSphere());
        }
        boolean done = initEmailCore();
        if (!done) {
            return false;
        }
        initPersonCore();
        return true;
    }

    private boolean initEmailCore() {
        String addrspec = StringUtils.cleanString(requestMap.getParameter(ADDRSPEC_PARAMNAME));
        if ((addrspec == null) || (addrspec.length() == 0)) {
            emailCore = null;
            return true;
        }
        try {
            emailCore = EmailCoreUtils.parse(addrspec);
            return true;
        } catch (ParseException pe) {
            setError("_ error.wrong.email", addrspec);
            return false;
        }
    }

    private void initPersonCore() throws BdfInstructionException {
        String surname = getMandatory(SURNAME_PARAMNAME);
        surname = StringUtils.cleanString(surname);
        String forename = StringUtils.cleanString(requestMap.getParameter(FORENAME_PARAMNAME));
        if (forename == null) {
            forename = "";
        }
        String nonlatin = StringUtils.cleanString(requestMap.getParameter(NONLATIN_PARAMNAME));
        if (nonlatin == null) {
            nonlatin = "";
        }
        boolean surnameFirst = requestMap.isTrue(SURNAMEFIRST_PARAMNAME);
        personCore = PersonCoreUtils.toPersonCore(surname, forename, nonlatin, surnameFirst);
    }

}
