/* BdfServer_Commands - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.sphere;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.namespaces.BdfUserSpace;
import fr.exemole.bdfserver.api.users.BdfUserPrefs;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.util.Set;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.sphere.Redacteur;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeBuilder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurTemplatesCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "RedacteurTemplates";
    public final static String COMMANDKEY = "_ USR-04";
    private BdfServerEditor bdfServerEditor;
    private BdfUserPrefs redacBdfUserPrefs;

    public RedacteurTemplatesCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        bdfServerEditor = bdfServer.getBdfServerEditor();
        saveKeys();
        bdfServerEditor.saveChanges();
        setDone("_ done.sphere.redacteurtemplates");
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        Redacteur redacteur = getMandatoryRedacteur();
        boolean isSame = BdfUserUtils.isSame(bdfUser, redacteur);
        if (!isSame) {
            checkSubsetAdminPermission(redacteur.getSphere());
        }
        if (isSame) {
            redacBdfUserPrefs = bdfUser.getPrefs();
        } else {
            redacBdfUserPrefs = bdfServer.createBdfUser(redacteur).getPrefs();
        }
        return true;
    }

    private void saveKeys() {
        saveTransformationKey(TransformationKey.COMPILATION_TRANSFORMATIONKEY);
        saveTransformationKey(TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY);
        saveTransformationKey(TransformationKey.INVERSETHESAURUS_TRANSFORMATIONKEY);
        for (Corpus corpus : fichotheque.getCorpusList()) {
            TransformationKey transformationKey = new TransformationKey(corpus.getSubsetKey());
            saveTransformationKey(transformationKey);
        }
    }

    private void saveTransformationKey(TransformationKey transformationKey) {
        String value = requestMap.getParameter(transformationKey.toString());
        if (value != null) {
            AttributeKey attributeKey = BdfUserSpace.toSimpleTemplateAttributeKey(transformationKey);
            if (value.equals(TemplateKey.DEFAULT_NAME)) {
                bdfServerEditor.removeAttribute(redacBdfUserPrefs, attributeKey);
            } else {
                Attribute attribute = AttributeBuilder.toAttribute(attributeKey, value);
                if (attribute == null) {
                    bdfServerEditor.removeAttribute(redacBdfUserPrefs, attributeKey);
                } else {
                    bdfServerEditor.putAttribute(redacBdfUserPrefs, attribute);
                }
            }
        }
        String prefix = transformationKey.toString() + "/";
        Set<String> nameSet = requestMap.getParameterNameSet();
        for (String paramName : nameSet) {
            if (paramName.startsWith(prefix)) {
                String extension = paramName.substring(prefix.length());
                String value2 = requestMap.getParameter(paramName);
                if (value2 != null) {
                    AttributeKey attributeKey = BdfUserSpace.toStreamTemplateAttributeKey(transformationKey, extension);
                    if (value2.equals(TemplateKey.DEFAULT_NAME)) {
                        bdfServerEditor.removeAttribute(redacBdfUserPrefs, attributeKey);
                    } else {
                        Attribute attribute = AttributeBuilder.toAttribute(attributeKey, value2);
                        if (attribute == null) {
                            bdfServerEditor.removeAttribute(redacBdfUserPrefs, attributeKey);
                        } else {
                            bdfServerEditor.putAttribute(redacBdfUserPrefs, attribute);
                        }
                    }
                }
            }
        }
    }

}
