/* BdfServer_Commands - Copyright (c) 2014-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.thesaurus;

import fr.exemole.bdfserver.api.BdfServer;
import static fr.exemole.bdfserver.api.instruction.BdfInstructionConstants.MOTCLE_OBJ;
import static fr.exemole.bdfserver.api.instruction.BdfInstructionConstants.THESAURUS_OBJ;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.util.LinkedHashSet;
import java.util.Set;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.thesaurus.ThesaurusEditor;
import net.fichotheque.tools.thesaurus.ChildrenReorderEngine;
import net.fichotheque.utils.ThesaurusUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class ChildrenReorderCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "ChildrenReorder";
    public final static String COMMANDKEY = "_ THS-21";
    public final static String CHILDREN_PARAMNAME = "children";
    private Thesaurus thesaurus;
    private Motcle motcle;
    private final Set<Motcle> newOrderSet = new LinkedHashSet<Motcle>();

    public ChildrenReorderCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.THESAURUS + "/" + COMMANDNAME);
        ThesaurusEditor thesaurusEditor = editSession.getFichothequeEditor().getThesaurusEditor(thesaurus.getSubsetKey());
        ChildrenReorderEngine engine = new ChildrenReorderEngine(thesaurusEditor);
        engine.reorder(motcle, newOrderSet);
        setDone("_ done.thesaurus.childrenreorder");
        putResultObject(THESAURUS_OBJ, thesaurus);
        if (motcle != null) {
            putResultObject(MOTCLE_OBJ, motcle);
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        thesaurus = getMandatoryThesaurus();
        checkSubsetAdminPermission(thesaurus);
        motcle = BdfInstructionUtils.getOptionnalMotcle(requestMap, thesaurus);
        String list = getMandatory(CHILDREN_PARAMNAME);
        String[] tokens = StringUtils.getLineTokens(list, StringUtils.EMPTY_EXCLUDE);
        boolean idalphaTest = ThesaurusUtils.isIdalphaType(thesaurus);
        int length = tokens.length;
        for (int i = 0; i < length; i++) {
            String token = tokens[i];
            Motcle child = null;
            if (idalphaTest) {
                child = thesaurus.getMotcleByIdalpha(token);
            } else {
                try {
                    int id = Integer.parseInt(token);
                    child = thesaurus.getMotcleById(id);
                } catch (NumberFormatException nfe) {
                }
            }
            if ((child != null) && (isParent(motcle, child))) {
                newOrderSet.add(child);
            }
        }
        return true;
    }

    private static boolean isParent(Motcle parent, Motcle child) {
        if (parent == null) {
            return (child.getParent() == null);
        } else if (child.getParent() == null) {
            return false;
        } else {
            return child.getParent().equals(parent);
        }

    }

}
