/* BdfServer_Commands - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.thesaurus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.croisement.CroisementEditor;
import net.fichotheque.include.IncludeKey;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.tools.croisement.CroisementChangeEngine;
import net.fichotheque.tools.croisement.LienBuffer;
import net.fichotheque.tools.parsers.croisement.LienBufferParser;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class MotcleIndexationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "MotcleIndexation";
    public final static String COMMANDKEY = "_ THS-13";
    private Motcle motcle;
    private List<CroisementByCorpus> croisementByCorpusList;

    public MotcleIndexationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        initEditSession(Domains.THESAURUS + "/" + COMMANDNAME);
        CroisementEditor croisementEditor = editSession.getFichothequeEditor().getCroisementEditor();
        List<IncludeKey> scopeList = new ArrayList<IncludeKey>();
        for (CroisementByCorpus croisementByCorpus : croisementByCorpusList) {
            scopeList.add(IncludeKey.newInstance(croisementByCorpus.corpus.getSubsetKey()));
        }
        CroisementChangeEngine croisementChangeEngine = CroisementChangeEngine.clearExistingEngine(motcle, scopeList);
        for (CroisementByCorpus croisementByCorpus : croisementByCorpusList) {
            List<LienBuffer> lienBufferList = croisementByCorpus.croisementList;
            int count = lienBufferList.size();
            for (int i = 0; i < count; i++) {
                LienBuffer lienBuffer = lienBufferList.get(i);
                croisementChangeEngine.addLien(lienBuffer);
            }
        }
        croisementEditor.updateCroisements(motcle, croisementChangeEngine.toCroisementChanges());
        setDone("_ done.thesaurus.motcleindexation");
        putResultObject(THESAURUS_OBJ, motcle.getThesaurus());
        putResultObject(MOTCLE_OBJ, motcle);
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        motcle = getMandatoryMotcle();
        checkSubsetAdminPermission(motcle.getThesaurus());
        PermissionSummary permissionSummary = getPermissionSummary();
        croisementByCorpusList = new ArrayList<CroisementByCorpus>();
        for (Corpus corpus : fichotheque.getCorpusList()) {
            if (permissionSummary.hasAccess(corpus)) {
                String param_value = requestMap.getParameter(corpus.getSubsetKey().toString());
                if (param_value != null) {
                    croisementByCorpusList.add(parse(corpus, param_value));
                }
            }
        }
        return true;
    }

    private CroisementByCorpus parse(Corpus corpus, String chaine) {
        String[] tokens = StringUtils.getTechnicalTokens(chaine, false);
        List<LienBuffer> lienBufferList = new ArrayList<LienBuffer>();
        for (String token : tokens) {
            try {
                LienBuffer lienBuffer = LienBufferParser.parseId(corpus, token, null, -1);
                lienBufferList.add(lienBuffer);
            } catch (ParseException pe) {

            }
        }
        return new CroisementByCorpus(corpus, lienBufferList);
    }


    private static class CroisementByCorpus {

        private final Corpus corpus;
        private final List<LienBuffer> croisementList;

        private CroisementByCorpus(Corpus corpus, List<LienBuffer> croisementList) {
            this.corpus = corpus;
            this.croisementList = croisementList;
        }

    }

}
