/* BdfServer_HtmlProducers - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.CorpusDomain;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.consumers.PageUnit;
import fr.exemole.bdfserver.html.jslib.CommonJsLibs;
import fr.exemole.bdfserver.tools.ui.components.UiComponentUtils;
import java.util.List;
import net.fichotheque.corpus.Corpus;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangsUtils;


/**
 *
 * @author Vincent Calame
 */
public class IncludeCommentFormHtmlProducer extends BdfServerHtmlProducer {

    private final PermissionSummary permissionSummary;
    private final Corpus corpus;

    public IncludeCommentFormHtmlProducer(ProducerParameters producerParameters, Corpus corpus) {
        this(producerParameters.getBdfServer(), producerParameters.getBdfUser(), producerParameters.getPermissionSummary(), corpus);
    }

    public IncludeCommentFormHtmlProducer(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, Corpus corpus) {
        super(bdfServer, bdfUser);
        addJsLib(CommonJsLibs.COMMANDTEST);
        addJsLib(CommonJsLibs.DEPLOY);
        addJsLib(CommonJsLibs.SUBSETCHANGE);
        addJsLib(CommonJsLibs.CODEMIRRORMODE);
        addThemeCss("corpus.css");
        this.permissionSummary = permissionSummary;
        this.corpus = corpus;
    }

    @Override
    public void printHtml() {
        UiComponents uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        List<CommentUi> list = UiComponentUtils.getCommentUiList(uiComponents);
        Lang[] langArray = LangsUtils.toArray(bdfServer.getLangConfiguration().getWorkingLangs());
        start();
        BdfHtmlUtils.printSubsetHeader(this, bdfServer, bdfUser, permissionSummary, corpus, CorpusDomain.INCLUDE_COMMENTFORM_PAGE);
        CorpusHtmlUtils.printCorpusToolbar(this, corpus, CorpusDomain.INCLUDE_COMMENTFORM_PAGE);
        printCommandMessageUnit();
        CommandBox commandBox = CommandBox.init()
                .action(Domains.CORPUS)
                .family("CRP")
                .page(CorpusDomain.INCLUDE_COMMENTFORM_PAGE)
                .mode(InteractionConstants.SUBUNIT_MODE);
        if (list.size() > 0) {
            this
                    .__(PageUnit.start("action-Comment", "_ title.corpus.comment_list"))
                    .DL("global-DL");
            for (CommentUi commentUi : list) {
                this
                        .DT()
                        .__(BdfHtmlUtils.printCommentCodeMirrorSpan(this, commentUi, workingLang))
                        ._DT()
                        .DD()
                        .__(CorpusHtmlUtils.printCommentChangeDetails(this, commandBox, corpus, commentUi, langArray))
                        .__(CorpusHtmlUtils.printCommentRemoveDetails(this, commandBox, corpus, commentUi))
                        ._DD();
            }
            this
                    ._DL()
                    .__(PageUnit.END);
        }
        CorpusCommandBoxUtils.printCommentCreationBox(this, commandBox.mode(""), corpus, langArray);
        end();
    }

}
