/* BdfServer_HtmlProducers - Copyright (c) 2013-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.sphere;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.domains.SphereDomain;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.jslib.CommonJsLibs;
import net.fichotheque.sphere.Redacteur;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurAvancedCommandsHtmlProducer extends BdfServerHtmlProducer {

    private final Redacteur redacteur;

    public RedacteurAvancedCommandsHtmlProducer(ProducerParameters producerParameters, Redacteur redacteur) {
        this(producerParameters.getBdfServer(), producerParameters.getBdfUser(), redacteur);
    }

    public RedacteurAvancedCommandsHtmlProducer(BdfServer bdfServer, BdfUser bdfUser, Redacteur redacteur) {
        super(bdfServer, bdfUser);
        addJsLib(CommonJsLibs.COMMANDTEST);
        addJsLib(CommonJsLibs.CODEMIRRORMODE);
        this.redacteur = redacteur;
        addThemeCss("sphere.css");
    }

    @Override
    public void printHtml() {
        start();
        SphereHtmlUtils.printRedacteurToolbar(this, SphereDomain.REDACTEUR_ADVANCEDCOMMANDS_PAGE, redacteur);
        printCommandMessageUnit();
        CommandBox commandBox = CommandBox.init()
                .action(Domains.SPHERE)
                .family("SPH")
                .page(SphereDomain.REDACTEUR_ADVANCEDCOMMANDS_PAGE);
        SphereCommandBoxUtils.printRedacteurAttributeChangeBox(this, commandBox, redacteur);
        end();
    }

}
