/* BdfServer_JsonProducers - Copyright (c) 2016-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.album;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.AlbumDomain;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import java.io.File;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.MessageLocalisation;


/**
 *
 * @author Vincent Calame
 */
public final class AlbumJsonProducerFactory {

    private AlbumJsonProducerFactory() {

    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws BdfInstructionException {
        String json = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        BdfUser bdfUser = parameters.getBdfUser();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        MessageLocalisation messageLocalisation = bdfServer.getL10nManager().getMessageLocalisation(bdfUser.getLangPreference());
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, messageLocalisation);
        switch (json) {
            case AlbumDomain.ILLUSTRATION_TMPFILE_JSON: {
                if (bdfCommandResult != null) {
                    File file = (File) bdfCommandResult.getResultObject(BdfInstructionConstants.TMPFILE_OBJ);
                    if (file != null) {
                        jsonProducer.add(new TmpFileJsonProperty(file));
                        return jsonProducer;
                    }
                }
                return null;
            }
            default:
                return null;
        }
    }

}
