/* BdfServer_JsonProducers - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.pioche;

import java.io.IOException;
import java.util.Collection;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.exportation.table.Cell;
import net.fichotheque.exportation.table.CellEngine;
import net.fichotheque.json.CellJson;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;


/**
 *
 * @author Vincent Calame
 */
public class FichesJsonProducer implements JsonProducer {

    private final Collection<FicheMeta> fiches;
    private final CellEngine cellEngine;


    public FichesJsonProducer(Collection<FicheMeta> fiches, CellEngine cellEngine) {
        this.fiches = fiches;
        this.cellEngine = cellEngine;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jw = new JSONWriter(appendable);
        jw.object();
        {
            jw.key("length")
                    .value(fiches.size());
            jw.key("array");
            jw.array();
            for (FicheMeta ficheMeta : fiches) {
                int ficheid = ficheMeta.getId();
                String corpusName = ficheMeta.getCorpus().getSubsetName();
                Cell[] cellArray = cellEngine.getCellArray(ficheMeta);
                jw.object();
                {
                    jw.key("corpus")
                            .value(corpusName);
                    jw.key("id")
                            .value(ficheid);
                    jw.key("code")
                            .value(corpusName + '/' + ficheid);
                    jw.key("title")
                            .value(ficheMeta.getTitre());
                    CellJson.cellArrayMappingProperty(jw, cellArray);
                }
                jw.endObject();
            }
            jw.endArray();
        }
        jw.endObject();
    }

}
