/* FichothequeLib_Tools - Copyright (c) 2008-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.conversion.fieldtofield;

import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.Fiche;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.FieldKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.MultiMessageHandler;


/**
 *
 * @author Vincent Calame
 */
public abstract class FieldToFieldConverter {

    protected final MultiMessageHandler messageHandler;

    protected FieldToFieldConverter(MultiMessageHandler messageHandler) {
        this.messageHandler = messageHandler;
    }

    protected void addWarning(String messageKey, Object messageValue) {
        messageHandler.addMessage("warning.conversion", LocalisationUtils.toMessage(messageKey, messageValue));
    }

    public static FieldToFieldConverter getInstance(CorpusField source, CorpusField destination, MultiMessageHandler messageHandler) {
        return getInstance(source.getFieldKey(), destination.getFieldKey(), messageHandler);
    }

    public static FieldToFieldConverter getInstance(FieldKey sourceKey, FieldKey destinationKey, MultiMessageHandler messageHandler) {
        int type = 0;
        if ((destinationKey.isProprieteFieldKey()) || (destinationKey.isInformationFieldKey())) {
            type = 1;
        } else if (destinationKey.isSectionFieldKey()) {
            type = 2;
        } else {
            throw new IllegalArgumentException("wrong destination : " + destinationKey.toString());
        }
        if (sourceKey.isProprieteFieldKey()) {
            if (type == 1) {
                return new ProprieteToFicheItemField(sourceKey, destinationKey, messageHandler);
            } else if (type == 2) {
                return new FicheItemFieldToFicheBlockField(sourceKey, destinationKey, messageHandler);
            }
        } else if ((sourceKey.isInformationFieldKey()) || (sourceKey.equals(FieldKey.REDACTEURS))) {
            if (type == 1) {
                return new InformationToFicheItemField(sourceKey, destinationKey, messageHandler);
            } else if (type == 2) {
                return new FicheItemFieldToFicheBlockField(sourceKey, destinationKey, messageHandler);
            }
        } else if (sourceKey.isSectionFieldKey()) {
            if (type == 1) {
                return new SectionToFicheItemField(sourceKey, destinationKey, messageHandler);
            } else if (type == 2) {
                return new SectionToFicheBlockField(sourceKey, destinationKey, messageHandler);
            }
        } else if (sourceKey.isSpecialFieldKey()) {
            return new SimpleToField(sourceKey, destinationKey, messageHandler);
        }
        throw new SwitchException("wrong category : " + sourceKey.toString());
    }

    public void convert(FicheMeta ficheMeta, Fiche fiche) {
        convert(ficheMeta, fiche, false);
    }

    public void convert(FicheMeta ficheMeta, Fiche fiche, boolean removeOldField) {
        String uri = ficheMeta.getCorpus().getSubsetName() + "/" + ficheMeta.getId();
        messageHandler.setCurrentURI(uri);
        convertContent(fiche, removeOldField);
    }

    abstract void convertContent(Fiche fiche, boolean removeOldField);

}
